@@ -17,33 +17,23 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
-
-#ifdef HAVE_LIBINPUT
-#include "libinput-properties.h"
-#endif /* HAVE_LIBINPUT */
-
-#include <glib.h>
-#include <gdk/gdk.h>
-#include <gdk/gdkx.h>
-#include <xfconf/xfconf.h>
-#include <libxfce4util/libxfce4util.h>
-#include <locale.h>
+#include "pointers-defines.h"
+#include "pointers.h"
 
 #include "common/debug.h"
-#include "pointers.h"
-#include "pointers-defines.h"
+
+#include <gdk/gdkx.h>
+#include <libxfce4util/libxfce4util.h>
+#include <xfconf/xfconf.h>
+
+#ifdef HAVE_LIBINPUT
+#include <libinput-properties.h>
+#endif /* HAVE_LIBINPUT */
+
+#include <locale.h>
 
 #define MAX_DENOMINATOR (100.00)
 
@@ -53,26 +43,33 @@
 #define DEVICE_ENABLED "Device Enabled"
 #endif /* XI_PROP_ENABLED */
 
-static void             xfce_pointers_helper_finalize                 (GObject            *object);
-static void             xfce_pointers_helper_syndaemon_stop           (XfcePointersHelper *helper);
-static void             xfce_pointers_helper_syndaemon_check          (XfcePointersHelper *helper);
-static void             xfce_pointers_helper_restore_devices          (XfcePointersHelper *helper,
-                                                                       XID                *xid);
-static void             xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
-                                                                       const gchar        *property_name,
-                                                                       const GValue       *value,
-                                                                       XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_finalize (GObject *object);
+static void
+xfce_pointers_helper_syndaemon_stop (XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper);
+static void
+xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
+                                      XID *xid);
+static void
+xfce_pointers_helper_channel_property_changed (XfconfChannel *channel,
+                                               const gchar *property_name,
+                                               const GValue *value,
+                                               XfcePointersHelper *helper);
 #ifdef DEVICE_HOTPLUGGING
-static GdkFilterReturn  xfce_pointers_helper_event_filter             (GdkXEvent          *xevent,
-                                                                       GdkEvent           *gdk_event,
-                                                                       gpointer            user_data);
+static GdkFilterReturn
+xfce_pointers_helper_event_filter (GdkXEvent *xevent,
+                                   GdkEvent *gdk_event,
+                                   gpointer user_data);
 #endif
 #if defined(DEVICE_PROPERTIES) || defined(HAVE_LIBINPUT)
-static void             xfce_pointers_helper_change_property          (XDeviceInfo        *device_info,
-                                                                       XDevice            *device,
-                                                                       Display            *xdisplay,
-                                                                       const gchar        *prop_name,
-                                                                       const GValue       *value);
+static void
+xfce_pointers_helper_change_property (XDeviceInfo *device_info,
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      const gchar *prop_name,
+                                      const GValue *value);
 #endif /* DEVICE_PROPERTIES || HAVE_LIBINPUT */
 
 
@@ -84,29 +81,28 @@ struct _XfcePointersHelperClass
 
 struct _XfcePointersHelper
 {
-    GObject  __parent__;
+    GObject __parent__;
 
     /* xfconf channel */
     XfconfChannel *channel;
 
 #ifdef DEVICE_PROPERTIES
-    GPid           syndaemon_pid;
+    GPid syndaemon_pid;
 #endif
 
 #ifdef DEVICE_HOTPLUGGING
     /* device presence event type */
-    gint           device_presence_event_type;
+    gint device_presence_event_type;
 #endif
 };
 
 typedef struct
 {
-    Display     *xdisplay;
-    XDevice     *device;
+    Display *xdisplay;
+    XDevice *device;
     XDeviceInfo *device_info;
-    gsize        prop_name_len;
-}
-XfcePointerData;
+    gsize prop_name_len;
+} XfcePointerData;
 
 
 
@@ -128,9 +124,9 @@ static void
 xfce_pointers_helper_init (XfcePointersHelper *helper)
 {
     XExtensionVersion *version = NULL;
-    Display           *xdisplay;
+    Display *xdisplay;
 #ifdef DEVICE_HOTPLUGGING
-    XEventClass        event_class;
+    XEventClass event_class;
 #endif
 
     /* get the default display */
@@ -166,7 +162,7 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
 
         /* monitor the channel */
         g_signal_connect (G_OBJECT (helper->channel), "property-changed",
-             G_CALLBACK (xfce_pointers_helper_channel_property_changed), helper);
+                          G_CALLBACK (xfce_pointers_helper_channel_property_changed), helper);
 
         /* launch syndaemon if required */
         xfce_pointers_helper_syndaemon_check (helper);
@@ -187,6 +183,9 @@ xfce_pointers_helper_init (XfcePointersHelper *helper)
         }
 #endif
     }
+
+    if (version)
+        XFree (version);
 }
 
 
@@ -206,10 +205,10 @@ static gboolean
 xfce_pointers_is_enabled (Display *xdisplay,
                           XDevice *device)
 {
-    Atom     prop, type;
-    gulong   n_items, bytes_after;
-    gint     rc, format;
-    guchar  *data;
+    Atom prop, type;
+    gulong n_items, bytes_after;
+    gint rc, format;
+    guchar *data;
     gboolean enabled;
 
     prop = XInternAtom (xdisplay, DEVICE_ENABLED, False);
@@ -234,10 +233,10 @@ static gboolean
 xfce_pointers_is_libinput (Display *xdisplay,
                            XDevice *device)
 {
-    Atom     prop, type;
-    gulong   n_items, bytes_after;
-    gint     rc, format;
-    guchar  *data;
+    Atom prop, type;
+    gulong n_items, bytes_after;
+    gint rc, format;
+    guchar *data;
 
     prop = XInternAtom (xdisplay, LIBINPUT_PROP_LEFT_HANDED, False);
     gdk_x11_display_error_trap_push (gdk_display_get_default ());
@@ -279,19 +278,19 @@ static void
 xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 {
 #ifdef DEVICE_PROPERTIES
-    Display     *xdisplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
     XDeviceInfo *device_list;
-    XDevice     *device;
-    gint         n, ndevices;
-    Atom         touchpad_type;
-    Atom         touchpad_off_prop;
-    Atom        *props;
-    gint         i, nprops;
-    gboolean     have_synaptics = FALSE;
-    gdouble      disable_duration;
-    gchar        disable_duration_string[64];
-    gchar       *args[] = { "syndaemon", "-i", disable_duration_string, "-K", "-R", NULL };
-    GError      *error = NULL;
+    XDevice *device;
+    gint n, ndevices;
+    Atom touchpad_type;
+    Atom touchpad_off_prop;
+    Atom *props;
+    gint i, nprops;
+    gboolean have_synaptics = FALSE;
+    gdouble disable_duration;
+    gchar disable_duration_string[64];
+    gchar *args[] = { "syndaemon", "-i", disable_duration_string, "-K", "-R", NULL };
+    GError *error = NULL;
 
     /* only stop a running daemon */
     if (!xfconf_channel_get_bool (helper->channel, "/DisableTouchpadWhileTyping", FALSE))
@@ -339,7 +338,7 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 
     XFreeDeviceList (device_list);
 
-    start_stop_daemon:
+start_stop_daemon:
 
     /* stop the daemon in any case */
     xfce_pointers_helper_syndaemon_stop (helper);
@@ -349,7 +348,7 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
         disable_duration = xfconf_channel_get_double (helper->channel,
                                                       "/DisableTouchpadDuration",
                                                       2.0);
-        setlocale(LC_NUMERIC, "C"); /* syndaemon needs a dot for the float. Nothing localized! */
+        setlocale (LC_NUMERIC, "C"); /* syndaemon needs a dot for the float. Nothing localized! */
         g_snprintf (disable_duration_string, sizeof (disable_duration_string),
                     "%.1f", disable_duration);
 
@@ -369,15 +368,15 @@ xfce_pointers_helper_syndaemon_check (XfcePointersHelper *helper)
 
 
 static gboolean
-xfce_pointers_helper_change_button_mapping_swap (guchar   *buttonmap,
-                                                 gshort    num_buttons,
-                                                 gint      id_1,
-                                                 gint      id_2,
-                                                 gboolean  reverse)
+xfce_pointers_helper_change_button_mapping_swap (guchar *buttonmap,
+                                                 gshort num_buttons,
+                                                 gint id_1,
+                                                 gint id_2,
+                                                 gboolean reverse)
 {
     gshort n;
-    gint   id_a = -1;
-    gint   id_b = -1;
+    gint id_a = -1;
+    gint id_b = -1;
 
     /* figure out the position of the id_1 and id_2 buttons in the map */
     for (n = 0; n < num_buttons; n++)
@@ -410,18 +409,18 @@ xfce_pointers_helper_change_button_mapping_swap (guchar   *buttonmap,
 
 static void
 xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
-                                            XDevice     *device,
-                                            Display     *xdisplay,
-                                            gint         right_handed,
-                                            gint         reverse_scrolling)
+                                            XDevice *device,
+                                            Display *xdisplay,
+                                            gint right_handed,
+                                            gint reverse_scrolling)
 {
-    XAnyClassPtr  ptr;
-    gshort        num_buttons = 0;
-    guchar       *buttonmap;
-    gboolean      map_changed = FALSE;
-    gint          n;
-    gint          right_button;
-    GString      *readable_map;
+    XAnyClassPtr ptr;
+    gshort num_buttons = 0;
+    guchar *buttonmap;
+    gboolean map_changed = FALSE;
+    gint n;
+    gint right_button;
+    GString *readable_map;
 
 #ifdef HAVE_LIBINPUT
     if (xfce_pointers_is_libinput (xdisplay, device))
@@ -462,7 +461,7 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
         }
 
         /* advance the offset */
-        ptr = (XAnyClassPtr) ((gchar *) ptr + ptr->length);
+        ptr = (XAnyClassPtr) (gpointer) ((gchar *) ptr + ptr->length);
     }
 
     if (num_buttons == 0)
@@ -505,6 +504,13 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
                                                              5 /* scroll down */,
                                                              !reverse_scrolling))
             map_changed = TRUE;
+
+        if (xfce_pointers_helper_change_button_mapping_swap (buttonmap, num_buttons,
+                                                             6 /* scroll left */,
+                                                             7 /* scroll right */,
+                                                             !reverse_scrolling))
+
+            map_changed = TRUE;
     }
 
     /* only set on changes */
@@ -529,7 +535,7 @@ xfce_pointers_helper_change_button_mapping (XDeviceInfo *device_info,
                         device_info->name);
     }
 
-    leave:
+leave:
 
     g_free (buttonmap);
 }
@@ -548,18 +554,18 @@ xfce_pointers_helper_gcd (gint num,
 
 static void
 xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
-                                      XDevice     *device,
-                                      Display     *xdisplay,
-                                      gint         threshold,
-                                      gdouble      acceleration)
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      gint threshold,
+                                      gdouble acceleration)
 {
-    XFeedbackState      *states, *pt;
-    gint                 num_feedbacks;
-    XPtrFeedbackControl  feedback;
-    gint                 n;
-    gulong               mask = 0;
-    gint                 num, denom, gcd;
-    gboolean             found = FALSE;
+    XFeedbackState *states, *pt;
+    gint num_feedbacks;
+    XPtrFeedbackControl feedback;
+    gint n;
+    gulong mask = 0;
+    gint num, denom, gcd;
+    gboolean found = FALSE;
 
 #ifdef HAVE_LIBINPUT
     if (xfce_pointers_is_libinput (xdisplay, device))
@@ -592,7 +598,7 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
         if (pt->class != PtrFeedbackClass)
         {
             /* advance the offset */
-            pt = (XFeedbackState *) ((gchar *) pt + pt->length);
+            pt = (XFeedbackState *) (gpointer) ((gchar *) pt + pt->length);
             continue;
         }
 
@@ -669,10 +675,10 @@ xfce_pointers_helper_change_feedback (XDeviceInfo *device_info,
 
 
 static void
-xfce_pointers_helper_change_mode (XDeviceInfo  *device_info,
-                                  XDevice      *device,
-                                  Display      *xdisplay,
-                                  const gchar  *mode_name)
+xfce_pointers_helper_change_mode (XDeviceInfo *device_info,
+                                  XDevice *device,
+                                  Display *xdisplay,
+                                  const gchar *mode_name)
 {
     gint mode;
 
@@ -700,7 +706,7 @@ xfce_pointers_helper_change_mode (XDeviceInfo  *device_info,
 static gchar *
 xfce_pointers_helper_device_xfconf_name (const gchar *name)
 {
-    GString     *string;
+    GString *string;
     const gchar *p;
 
     /* NOTE: this function exists in both the dialog and
@@ -733,31 +739,33 @@ xfce_pointers_helper_device_xfconf_name (const gchar *name)
 
 #if defined(DEVICE_PROPERTIES) || defined(HAVE_LIBINPUT)
 static void
-xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
-                                      XDevice      *device,
-                                      Display      *xdisplay,
-                                      const gchar  *prop_name,
+xfce_pointers_helper_change_property (XDeviceInfo *device_info,
+                                      XDevice *device,
+                                      Display *xdisplay,
+                                      const gchar *prop_name,
                                       const GValue *value)
 {
-    Atom         *props;
-    gint          n, n_props;
-    Atom          prop;
-    gchar        *atom_name;
-    Atom          type;
-    gint          format;
-    gulong        n_items, bytes_after, i;
-    gulong        n_succeeds;
-    Atom          float_atom;
-    GPtrArray    *array = NULL;
-    int           rc;
+    Atom *props;
+    gint n, n_props;
+    Atom prop;
+    gchar *atom_name;
+    Atom type;
+    gint format;
+    gulong n_items, bytes_after, i;
+    gulong n_succeeds;
+    Atom float_atom;
+    GPtrArray *array = NULL;
+    int rc;
     const GValue *val;
-    union {
+    union
+    {
         guchar *c;
         gshort *s;
-        glong  *l;
-        float  *f;
-        Atom   *a;
+        glong *l;
+        float *f;
+        Atom *a;
     } data;
+    guchar *allocated_data = NULL;
 
     /* assuming the device property never contained underscores... */
     atom_name = g_strdup (prop_name);
@@ -776,8 +784,8 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
      * see: https://bugs.freedesktop.org/show_bug.cgi?id=89296
      * and: http://lists.x.org/archives/xorg-devel/2015-February/045716.html
      */
-    if (prop != XInternAtom (xdisplay, DEVICE_ENABLED, True) &&
-        !xfce_pointers_is_enabled (xdisplay, device))
+    if (prop != XInternAtom (xdisplay, DEVICE_ENABLED, True)
+        && !xfce_pointers_is_enabled (xdisplay, device))
         return;
 #endif /* HAVE_LIBINPUT */
 
@@ -806,17 +814,31 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
                     || G_VALUE_HOLDS_DOUBLE (value)))
             {
                 /* only 1 items to set */
-                val = value;
             }
             else if (G_VALUE_TYPE (value) == G_TYPE_PTR_ARRAY)
             {
                 array = g_value_get_boxed (value);
-                if (array->len != n_items)
+                /* LibInput array properties can have dynamic number of items.
+                   Do not enforce equal lengths, set as many items as defined in config
+                   and re-allocate the data array, if it is too small. */
+                if (array->len > n_items)
                 {
-                    g_critical ("Nr device property items (%ld) and xfconf value (%d) differ",
-                                n_items, array->len);
-                    break;
+                    switch (format)
+                    {
+                        case 8: allocated_data = (guchar *) g_new (guchar, array->len); break;
+                        case 16: allocated_data = (guchar *) g_new (ushort, array->len); break;
+                        case 32: allocated_data = (guchar *) g_new (ulong, array->len); break;
+                        default: allocated_data = NULL;
+                    }
+                    XFree (data.c);
+                    data.c = allocated_data;
+                    if (!allocated_data)
+                    {
+                        g_critical ("Unknown format %d for integer", format);
+                        break;
+                    }
                 }
+                n_items = array->len;
             }
             else
             {
@@ -863,7 +885,7 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
                 {
                     /* Xorg actually uses sizeof(long) bytes per element if format == 32 */
                     /* See https://gitlab.freedesktop.org/xorg/app/xinput/-/blob/cef07c0c8280d7e7b82c3bcc62a1dfbe8cc43ff8/src/property.c#L80 */
-                    *(float*) &data.l[i] = (float) g_value_get_double (val);
+                    *(float *) &data.l[i] = (float) g_value_get_double (val);
                 }
                 else
                 {
@@ -894,7 +916,9 @@ xfce_pointers_helper_change_property (XDeviceInfo  *device_info,
             }
         }
 
-        if (data.c)
+        if (allocated_data)
+            g_free (allocated_data);
+        else if (data.c)
             XFree (data.c);
 
         break;
@@ -912,7 +936,7 @@ xfce_pointers_helper_change_properties (gpointer key,
                                         gpointer user_data)
 {
     XfcePointerData *pointer_data = user_data;
-    const gchar     *prop_name = ((gchar *) key) + pointer_data->prop_name_len;
+    const gchar *prop_name = ((gchar *) key) + pointer_data->prop_name_len;
 
     xfce_pointers_helper_change_property (pointer_data->device_info,
                                           pointer_data->device,
@@ -925,21 +949,21 @@ xfce_pointers_helper_change_properties (gpointer key,
 
 static void
 xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
-                                      XID                *xid)
+                                      XID *xid)
 {
-    Display         *xdisplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
-    XDeviceInfo     *device_list, *device_info;
-    gint             n, ndevices;
-    XDevice         *device;
-    gchar           *device_name;
-    gchar            prop[256];
-    gboolean         right_handed;
-    gboolean         reverse_scrolling;
-    gint             threshold;
-    gdouble          acceleration;
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    XDeviceInfo *device_list, *device_info;
+    gint n, ndevices;
+    XDevice *device;
+    gchar *device_name;
+    gchar prop[256];
+    gboolean right_handed;
+    gboolean reverse_scrolling;
+    gint threshold;
+    gdouble acceleration;
 #ifdef DEVICE_PROPERTIES
-    GHashTable      *props;
-    XfcePointerData  pointer_data;
+    GHashTable *props;
+    XfcePointerData pointer_data;
 #endif
 
     gdk_x11_display_error_trap_push (gdk_display_get_default ());
@@ -1004,7 +1028,7 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
 
         /* read mode settings */
         g_snprintf (prop, sizeof (prop), "/%s/Mode", device_name);
-        mode =  xfconf_channel_get_string  (helper->channel, prop, NULL);
+        mode = xfconf_channel_get_string (helper->channel, prop, NULL);
 
         if (mode != NULL)
         {
@@ -1041,24 +1065,24 @@ xfce_pointers_helper_restore_devices (XfcePointersHelper *helper,
 
 
 static void
-xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
-                                               const gchar        *property_name,
-                                               const GValue       *value,
+xfce_pointers_helper_channel_property_changed (XfconfChannel *channel,
+                                               const gchar *property_name,
+                                               const GValue *value,
                                                XfcePointersHelper *helper)
 {
-    Display      *xdisplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default());
-    XDeviceInfo  *device_list, *device_info;
-    XDevice      *device;
-    gint          n, ndevices;
-    gchar       **names;
-    gchar        *device_name;
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    XDeviceInfo *device_list, *device_info;
+    XDevice *device;
+    gint n, ndevices;
+    gchar **names;
+    gchar *device_name;
 
     if (G_UNLIKELY (property_name == NULL))
-         return;
+        return;
 
     /* check the daemon status */
-    if ((strcmp (property_name, "/DisableTouchpadWhileTyping") == 0) ||
-        (strcmp (property_name, "/DisableTouchpadDuration") == 0))
+    if (strcmp (property_name, "/DisableTouchpadWhileTyping") == 0
+        || strcmp (property_name, "/DisableTouchpadDuration") == 0)
     {
         xfce_pointers_helper_syndaemon_check (helper);
         return;
@@ -1157,12 +1181,12 @@ xfce_pointers_helper_channel_property_changed (XfconfChannel      *channel,
 #ifdef DEVICE_HOTPLUGGING
 static GdkFilterReturn
 xfce_pointers_helper_event_filter (GdkXEvent *xevent,
-                                   GdkEvent  *gdk_event,
-                                   gpointer   user_data)
+                                   GdkEvent *gdk_event,
+                                   gpointer user_data)
 {
-    XEvent                     *event = xevent;
+    XEvent *event = xevent;
     XDevicePresenceNotifyEvent *dpn_event = xevent;
-    XfcePointersHelper         *helper = XFCE_POINTERS_HELPER (user_data);
+    XfcePointersHelper *helper = XFCE_POINTERS_HELPER (user_data);
 
     if (event->type == helper->device_presence_event_type)
     {
